class Config:
    def __init__(self):
        self.local_ip = '127.0.0.1'
        self.local_port = 7788
        self.client_name = 'xyz'
        self.regist_server_url = r'https://www.baidu.com/1'
        self.heart_beat_url = r'https://www.baidu.com/'
        self.result_url = r'https://www.baidu.com/'
        self.http_request_timeout = 1
        self.max_thread = 100
        self.max_task = 2000
        self.test_script_dir = r'D:\1\ts'
        self.py_timeout = 15
        self.robot_url = ''
        self.work_path = ''


config = Config()


class FMap:
    def __init__(self):
        self.func_map = {}
        # 例子
        # func_map = {
        #     'omc': {
        #         '/omc/mark/path1': [
        #             {'函数对象': {
        #                 'timeout': 2,
        #                 'name': '场景描述123'
        #             }},
        #             {'函数对象': {
        #                 'timeout': 5,
        #                 'name': '场景描述321'
        #             }},
        #         ]
        #     }
        # }

    def add_map(self, app, path, func, timeout, name):
        if app not in self.func_map:
            self.func_map[app] = {path: [{func: {'timeout': timeout, 'name': name}}]}
        else:
            t_dict = self.func_map[app]
            if path not in t_dict:
                self.func_map[app][path] = [{func: {'timeout': timeout, 'name': name}}]
            else:
                self.func_map[app][path].append({func: {'timeout': timeout, 'name': name}})

    def get_map(self):
        return self.func_map


fm = FMap()


py_name = ''



import time
from loguru import logger
from script_client.const import fm



def register(app='',path='',name='',timeout=0):
    '''
    注册为回调函数
    :param app: app_code
    :param path: path
    :param timeout: timeout 本期没做无效
    :return:
    '''
    def deco(func):
        fm.add_map(app=app.strip(),path=path.strip(),func=func,
                   name=name,timeout=timeout)
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)
        return wrapper
    return deco



def print_run_time(func):
    def wrapper(*args, **kwargs):
        t1 = time.time()
        var = func(*args, **kwargs)
        t2 = time.time()
        t = '%.6f' % (t2 - t1)
        logger.info(f"函数 {func.__name__} 运行消耗了 {t} 秒")
        return var
    return wrapper


from setuptools import setup, find_packages
import shutil


try:
    shutil.rmtree("build")
    shutil.rmtree("dist")
except:
    pass


setup(name='script_client',
      version='1.5.0',
      author='TDC',
      author_email='tiandachun@meicai.cn',
      description="script client",
      long_description='''script client''',
      install_requires=['JPype1>=0.7.4',
                        "loguru>=0.3.2",
                        "requests>=2.22.0",
                        "tornado>=6.1",
                        "APScheduler>=3.7.0",
                        "psutil>=5.8.0",
                        ],
      license="MIT",
      packages=find_packages(),
      include_package_data=True,
      data_files=[
          ('script_client/static', ['script_client/static/layui.js']),
          ('script_client/static/js', ['script_client/static/js/ace.js', 'script_client/static/js/ext-language_tools.js',
                                    'script_client/static/js/mode-python.js','script_client/static/js/theme-eclipse.js']),
          ('script_client/static/js/snippets', ['script_client/static/js/snippets/python.js']),
          ('script_client/static/font',['script_client/static/font/iconfont.eot','script_client/static/font/iconfont.svg',
                                        'script_client/static/font/iconfont.ttf','script_client/static/font/iconfont.woff',
                                        'script_client/static/font/iconfont.woff2']),
          ('script_client/static/css', ['script_client/static/css/layui.css']),
          ('script_client/static/css/modules', ['script_client/static/css/modules/code.css']),
          ('script_client/static/css/modules/laydate/default', ['script_client/static/css/modules/laydate/default/laydate.css']),
          ('script_client/static/css/modules/layer/default', ['script_client/static/css/modules/layer/default/icon.png',
                                                              'script_client/static/css/modules/layer/default/icon-ext.png',
                                                              'script_client/static/css/modules/layer/default/layer.css',
                                                              'script_client/static/css/modules/layer/default/loading-0.gif',
                                                              ]),
          ('script_client/templates', ['script_client/templates/base.html','script_client/templates/file_manage.html',
                                       'script_client/templates/sys_manage.html']),
      ]
      )



'''
common method
'''

import os
import json
import socket
import shutil
from loguru import logger
from script_client.const import config,fm
from script_client.common.my_requests import MyRequests


def find_script():
    l=[]
    if os.path.exists(os.sep.join([config.work_path,'test_script'])):
        for root, dirs, files in os.walk(os.sep.join([config.work_path,'test_script'])):
            for file in files:
                if os.path.splitext(file)[1] == '.py':
                    l.append(os.path.splitext(file)[0])
        return l
    else:
        return False

def save_script(f_path,f_text):
    try:
        with open(f_path, 'w', encoding='utf-8') as f:
            f.write(f_text)
        return True
    except Exception as e:
        return e

def upload_script(f_path,files):
    try:
        for meta in files:
            with open(f_path, 'wb+') as f:
                f.write(meta['body'])
        return True
    except Exception as e:
        return e

def replace_str(s):
    s = s.replace('\r\n', '')
    s = s.replace('\n', '')
    return s

def get_tree():
    tree = [
        {
            "title": "test_script",
            'spread': 'true',
            # "children": [
            #     {
            #         "title": "西安",
            #     },
            #     {
            #         "title": "延安",
            #     }
            # ]
        }
    ]
    children = []
    l = find_script()
    if l != False:
        for i in l:
            children.append({'title': i})
    tree[0]['children'] = children
    return tree

def get_local_ip():
    local_ip = ""
    try:
        socket_objs = [socket.socket(socket.AF_INET, socket.SOCK_DGRAM)]
        ip_from_ip_port = [(s.connect(("8.8.8.8", 53)), s.getsockname()[0], s.close()) for s in socket_objs][0][1]
        ip_from_host_name = [ip for ip in socket.gethostbyname_ex(socket.gethostname())[2] if
                             not ip.startswith("127.")][:1]
        local_ip = [l for l in (ip_from_ip_port, ip_from_host_name) if l][0]
    except (Exception) as e:
        print("get_local_ip found exception : %s" % e)
    return local_ip if ("" != local_ip and None != local_ip) else socket.gethostbyname(socket.gethostname())


def is_none(kwargs):
    '''
    0、空、None、False 外都算 True
    :param kwargs:
    :return:
    '''
    ll = []
    for k, v in kwargs.items():
        ll.append(v)
    if all(ll):
        return False
    return True


def del_path(path):
    '''
    删文件夹，以及文件夹内的文件
    :param path:
    :return:
    '''
    try:
        shutil.rmtree(path)
    except Exception as e:
        logger.error(f"清空 {path} 出错 ：{e}")


def del_all_file(path):
    '''
    删目录下所有文件，包括子目录内
    :param path:
    :return:
    '''
    for i in os.listdir(path):
        path_file = os.sep.join([path, i])
        if os.path.isfile(path_file):
            os.remove(path_file)
        else:
            del_all_file(path_file)


def robot_send_message(msg_type='text', message=''):
    try:
        if config.robot_url != '':
            mr = MyRequests(timeout=config.http_request_timeout)
            url = config.robot_url
            json = {
                "msg_type": msg_type,
                "content": {
                    "text": message
                }
            }
            rep = mr.run(method='post', url=url, json=json)
            if isinstance(rep, str) or rep.status_code != 200 or rep.json()['StatusMessage'] != 'success':
                return False
            return True
    except Exception as e:
        logger.error(f"{e}")
        return False


def regist_app():
    try:
        data = {
            'is_quanliang': 0,  # 0重启服务全量更新  非0追加
            'registry': []
        }
        for k, v in fm.get_map().items():
            t_d = {
                'scriptclient_name': config.client_name,
                'app_code': '',
                'ip': config.local_ip,
                'port': str(config.local_port),
                'path': []
            }
            t_d['app_code'] = k
            for kk, vv in v.items():
                t_l = []
                for vvv in vv:
                    for kkkk, vvvv in vvv.items():
                        t_j = {'name':vvvv['name'],'value':kkkk.__name__}
                        t_l.append(t_j)
                tt_d = {'moniter_path': kk, 'function': t_l}
                t_d['path'].append(tt_d)
            data['registry'].append(t_d)
        url = config.regist_server_url
        mr = MyRequests(config.http_request_timeout)
        # logger.info(data)
        rep = mr.run(url=url, method='post', json=data)
        if isinstance(rep, str) or rep.status_code != 200:
            msg = f"注册失败 接口返回 = {rep}"
            logger.error(msg)
            return msg
        else:
            # logger.info(f"注册成功--{data}")
            # logger.info(f"{rep}")
            return True
    except Exception as e:
        logger.error(f'注册失败:{e}')
        return f'注册失败:{e}'


def str_to_dict(str):
    str = str.replace(r'\r\n', '')
    str = str.replace("'", '"')
    return json.loads(str)


if "__main__" == __name__:
    # del_path(r'D:\1\2 python code\script_client\test_script\1')
    pass



import os
import sys
import copy
import json
import psutil
import platform
import tornado.web
import tornado.ioloop
import tornado.httpserver
import tornado.gen as gen
from loguru import logger
from script_client import file_manage
from script_client import run_script
from script_client.const import config, fm
from script_client.deco import print_run_time
from script_client.common.common_method import save_script,upload_script,get_tree


class ScriptManageHandler(tornado.web.RequestHandler):
    def get(self):
        self.render('file_manage.html', f_tree=get_tree())

class ListenerHandler(tornado.web.RequestHandler):
    def set_default_headers(self):
        self.set_header('Content-Type', 'application/json')

    # @print_run_time
    @gen.coroutine
    def post(self):
        try:
            data = json.loads(self.request.body)
            if not all((data['app_code'], data['moniter_path'], data['http_body'])):
                logger.error(f"有参数为空 data={data}")
                self.write({'code': 0, 'msg': f"有参数为空"})
                return
            fun_map = fm.get_map()
            t_l = []
            if data['app_code'] in fun_map:
                if data['moniter_path'] in fun_map[data['app_code']]:
                    for fun_name in data['fun_name']:
                        for f in fun_map[data['app_code']][data['moniter_path']]:
                            for k, v in f.items():
                                if k.__name__ == fun_name:
                                    data['func_obj'] = [k, v]
                                    t_l.append(copy.deepcopy(data))
                    if t_l:
                        if run_script.q.qsize() + len(t_l) > config.max_task:
                            logger.warning(f"{config.local_ip}:{config.local_port}的队列已满")
                            self.write({'code': 0, 'msg': f"{config.local_ip}:{config.local_port}的队列已满"})
                            return
                        for t_data in t_l:
                            run_script.q.put(t_data)
                            # logger.debug(f"q队列长度 = {run_script.q.qsize()}")
                        self.write({'code': 1, 'msg': 'success'})
                        return
            logger.info(f"没有命中函数，现在注册的函数有 {fun_map}")
            self.write({'code': 0, 'msg': '没有命中函数'})
        except Exception as e:
            self.write({'code': 0, 'msg': f'{e}'})

class GetScriptHandler(tornado.web.RequestHandler):
    def post(self):
        try:
            data = json.loads(self.request.body)
            f_name = data['file_name'] + '.py'
            f_path = os.sep.join([config.work_path, 'test_script',f_name])
            if data['file_name'] == 'test_script':
                self.write('左树选择')
                return
            if os.path.exists(f_path):
                with open(f_path,'r',encoding='utf-8') as f:
                    self.write(f.read())
        except Exception as e:
            logger.error(str(e))

class SaveScriptHandler(tornado.web.RequestHandler):
    def post(self):
        try:
            data = json.loads(self.request.body)
            f_name = data['file_name'] + '.py'
            f_text = data['text']
            f_path = os.sep.join([config.work_path, 'test_script', f_name])
            ret = save_script(f_text=f_text,f_path=f_path)
            if ret != True:
                self.write({'code':0,'msg':ret})
                return
            self.write({'code':1,'msg':'保存成功'})
        except Exception as e:
            logger.error(str(e))
            self.write({'code': 0, 'msg': e})

class UploadScriptHandler(tornado.web.RequestHandler):
    def post(self):
        try:
            script_file = self.request.files.get('file', None)
            if script_file != None:
                f_path = os.sep.join([config.work_path, 'test_script', script_file[0].filename])
                ret = upload_script(f_path=f_path,files=script_file)
                if ret != True:
                    self.write({'code':0,'msg':ret})
                    return
                self.write({'code':1,'msg':'上传成功'})
        except Exception as e:
            logger.error(str(e))
            self.write({'code': 0, 'msg': e})

class GetTreeHandler(tornado.web.RequestHandler):
    def post(self):
        try:
            d = {'tree':get_tree()}
            print(d)
            self.write(d)
        except Exception as e:
            logger.error(str(e))
            self.write({'code': 0, 'msg': e})

def run_web_server():
    logger.info(f"启动webserver...")
    handlers = [
        (r"/", ScriptManageHandler),
        (r"/get_tree", GetTreeHandler),
        (r"/receive_data", ListenerHandler),
        (r"/get_script_text", GetScriptHandler),
        (r"/save_script_text", SaveScriptHandler),
        (r"/upload_script_text", UploadScriptHandler),
    ]
    setting = dict(
        template_path=os.path.join(os.path.dirname(__file__), "templates"),
        static_path=os.path.join(os.path.dirname(__file__), "static"),
    )
    app = tornado.web.Application(handlers, **setting)
    httpServer = tornado.httpserver.HTTPServer(app)
    httpServer.bind(int(config.local_port))
    if (platform.system() != "Windows"):
        httpServer.start(1)
    else:  # 目前情况一个实例绝对够了
        httpServer.start(1)
    tornado.ioloop.IOLoop.current().start()


def _kill_terminate(p):
    # p.terminate()
    p.kill()


def stop_web_server():
    if os.path.exists('pid.log') == False:
        logger.info('没有pid.log')
        sys.exit(0)
    with open("pid.log", 'r') as fp:
        pid = int(fp.read())
    try:
        pidc = psutil.Process(pid)
        # ppid = pid.parent()
        # _kill_terminate(ppid)
        _kill_terminate(pidc)
    except Exception as e:
        logger.error(f"{e}")


if __name__ == "__main__":
    run_web_server()
